@@ -39,7 +39,10 @@ if HAS_UNAME:
     CPU_PPC64 = (_machine in ('ppc64', 'ppc64le'))
     CPU_I386 = (_machine in ("i386", "i686"))    # compatible Intel 32 bits
     CPU_X86_64 = (_machine in ("x86_64", "amd64"))  # compatible Intel 64 bits
-    CPU_ARM = _machine.startswith('arm')
+    CPU_ARM32 = _machine.startswith('arm')
+    CPU_AARCH64 = (_machine == 'aarch64')
+    CPU_RISCV32 = (_machine == 'riscv32')
+    CPU_RISCV64 = (_machine == 'riscv64')
     del _machine
 else:
     # uname() fallback for Windows
@@ -48,7 +51,10 @@ else:
     CPU_PPC64 = False
     CPU_I386 = False
     CPU_X86_64 = False
-    CPU_ARM = False
+    CPU_ARM32 = False
+    CPU_AARCH64 = False
+    CPU_RISCV32 = False
+    CPU_RISCV64 = False
     bits, linkage = architecture()
     if bits == '32bit':
         CPU_I386 = True
@@ -59,3 +65,5 @@ else:
 
 CPU_INTEL = (CPU_I386 or CPU_X86_64)
 CPU_POWERPC = (CPU_PPC32 or CPU_PPC64)
+CPU_ARM = (CPU_ARM32 or CPU_AARCH64)
+CPU_RISCV = (CPU_RISCV32 or CPU_RISCV64)
