@@ -4,11 +4,12 @@ from datetime import datetime, timedelta
 from os import getenv, access, X_OK, pathsep, getcwd
 from os.path import join as path_join, isabs, dirname, normpath
 
+
 def dumpRegs(log, regs):
     """
     Dump all registers using log callback (write one line).
     """
-    width = max( len(name) for name, type in regs._fields_ )
+    width = max(len(name) for name, type in regs._fields_)
     name_format = "%% %us" % width
     for name, type in regs._fields_:
         value = getattr(regs, name)
@@ -21,6 +22,7 @@ def dumpRegs(log, regs):
             value = formatWordHex(value)
         log("%s = %s" % (name, value))
 
+
 def readBits(value, bitmasks):
     """
     Extract bits from the integer value using a list of bit masks.
@@ -33,6 +35,8 @@ def readBits(value, bitmasks):
     ...
     >>> readBits(5, bitmask)
     ['exec', 'read']
+    >>> readBits(12, bitmask)
+    ['read', '8']
     """
     bitset = []
     for mask, item in bitmasks:
@@ -40,8 +44,11 @@ def readBits(value, bitmasks):
             continue
         bitset.append(item)
         value = value & ~mask
+    if value:
+        bitset.append(str(value))
     return bitset
 
+
 def formatBits(value, bitmasks, empty_text=None, format_value=str):
     """
     Format a value using a bitmask: see readBits() functions.
@@ -70,11 +77,14 @@ def formatBits(value, bitmasks, empty_text=None, format_value=str):
         else:
             return str(value)
 
-LOCAL_TIMEZONE_OFFSET = datetime.fromtimestamp(0) - datetime.utcfromtimestamp(0)
+
+LOCAL_TIMEZONE_OFFSET = datetime.fromtimestamp(
+    0) - datetime.utcfromtimestamp(0)
 
 # Start of UNIX timestamp (Epoch): 1st January 1970 at 00:00
 UNIX_TIMESTAMP_T0 = datetime(1970, 1, 1)
 
+
 def timestampUNIX(value, is_local):
     """
     Convert an UNIX (32-bit) timestamp to datetime object. Timestamp value
@@ -93,6 +103,7 @@ def timestampUNIX(value, is_local):
         timestamp += LOCAL_TIMEZONE_OFFSET
     return timestamp
 
+
 def locateProgram(program):
     """
     Locate a program using the PATH environment variable. Return the
@@ -116,6 +127,7 @@ def locateProgram(program):
             return filename
     return program
 
+
 def minmax(min_value, value, max_value):
     """
     Restrict value to [min_value; max_value]
@@ -129,6 +141,7 @@ def minmax(min_value, value, max_value):
     """
     return min(max(min_value, value), max_value)
 
+
 def inverseDict(data):
     """
     Inverse a dictionary.
@@ -141,3 +154,14 @@ def inverseDict(data):
         result[value] = key
     return result
 
+
+def signal_to_exitcode(signum):
+    """
+    Converts a signal number to an exit code.
+    UNIX: https://www.gnu.org/software/bash/manual/html_node/Exit-Status.html
+
+    >>> import signal
+    >>> signal_to_exitcode(signal.SIGQUIT)
+    131
+    """
+    return 128 + signum
